#define F_CPU 12000000L
#include <avr/io.h>
#include <util/delay.h>
#include <stdlib.h>
#define TRUE 1
#define FALSE 0 
/*

Button 1 2 3 LED 1 2 3 is from LEFT to RIGHT, yellow side.

PB0 = Button1_P1
PB2 = Button2_P1
PB4 = Button3_P1

PB5 = BCDSeven0_P1
PB3 = BCDSeven1_P1
PB1 = BCDSeven2_P1

PC0 = Button1_P2
PC2 = Button2_P2
PC4 = Button3_P2

PC5 = BCDSeven0_P2
PC3 = BCDSeven1_P2
PC1 = BCDSeven2_P2

PD5 = LED_DecoderBit0 (A)
PD0 = LED_DecoderBit1 (B)
PD6 = Sound
PD1 = Mode

*/

//DDR 1 is out , 0 is in

uint8_t B_in(uint8_t pin)
{
	DDRB &= ~(1<<pin);
	PORTB |= 1<<pin;
	return !((PINB >> pin) & 1);
}
uint8_t C_in(uint8_t pin)
{
	DDRC &= ~(1<<pin);
	PORTC |= 1<<pin;
	return !((PINC >> pin) & 1);
}
uint8_t D_in(uint8_t pin)
{
	DDRD &= ~(1<<pin);
	PORTD |= 1<<pin;
	return !((PIND >> pin) & 1);
}
void InitDDR()
{

	DDRB |= 0b00101010; //PB5,PB3,PB1 (BCDSeven P1) out
	DDRB &= 0b11101010; //PB0,PB2,PB4 (Button P1) in
	PORTB |= 0b00010101; //PB0,PB2,PB4 pull up when Z
	PORTB &= 0b11010101; //PB5,PB3,PB1 out 0

	DDRC |= 0b00101010; //PC5,PC3,PC1 (BCDSeven P2) out
	DDRC &= 0b11101010; //PC0,PC2,PC4 (Button P2) in
	PORTC |=0b00010101; //PC0,PC2,PC4 pull up when Z
	PORTC &= 0b11010101; //PC5,PC3,PC1 out 0 by default

	DDRD |= 0b01100001; //PD5,PD0,PD6 (LED Decoder , Sound) out
	DDRD &= 0b11111101; // PD1 (Mode) in
	PORTD |= 0b00000010; //PD1 pull up when Z 
	PORTD &= 0b10011110; //PD6 sound out 0 / PD5,PD0 out 0 both (NO LED)
}

int ReceiveMode()
{
	if(D_in(1)) //dpdt switch
	{
		return TRUE; //instant mode
	}
	else
	{
		return FALSE;
	}
	return FALSE;
}

void Sound1(int ms)
{

	while(ms>=0)
	{
		PORTD |= 0b01000000;
		_delay_ms(3);
		PORTD &= 0b10111111;
		_delay_ms(3);
		ms--;
	}
	
}
void Sound2(int ms)
{
	while(ms>=0)
	{
		PORTD |= 0b01000000;
		_delay_ms(7);
		PORTD &= 0b10111111;
		_delay_ms(7);
		ms--;
	}
}

void Sound_Start()
{
	Sound1(10);
	_delay_ms(200);
	Sound1(7);
	_delay_ms(7);
	Sound1(7);
	_delay_ms(7);
	Sound1(7);
	_delay_ms(7);
	Sound1(60);
}

void Sound_Appear()
{
	Sound1(5);
}

void Sound_Damage()
{
	Sound1(5);
	_delay_ms(5);
	Sound1(5);
}

void Sound_Commit()
{
	Sound2(7);
}

void Sound_Ending()
{
	Sound1(10);
	_delay_ms(120);
	Sound1(10);
	_delay_ms(120);
	Sound1(10);
	_delay_ms(120);
	Sound1(10);
	_delay_ms(120);
	Sound1(10);

	_delay_ms(260);
	Sound2(15);
	_delay_ms(260);
	
	Sound1(10);
	_delay_ms(120);
	Sound1(10);
	_delay_ms(120);
	Sound1(10);
	_delay_ms(120);
	Sound1(10);
	_delay_ms(120);
	Sound1(10);
}

int GetButtonInputP1()
{
	if(B_in(0)) //Pressed P1 Btn1
	{
		return 1;
	}
	if(B_in(2)) //Pressed P1 Btn2
	{
		return 2;
	}
	if(B_in(4)) //Pressed P1 Btn3
	{
		return 3;
	}
	return 0;
}

int GetButtonInputP2()
{
	if(C_in(0)) //Pressed P2 Btn1
	{
		return 1;
	}
	if(C_in(2)) //Pressed P2 Btn2
	{
		return 2;
	}
	if(C_in(4)) //Pressed P2 Btn3
	{
		return 3;
	}
	return 0;
}

void Wait_LED(int instantmode,int* p1_commit,int* p2_commit)
{
	int mstowait = 0;
	if(instantmode)
	{
		mstowait = (rand()% 3000) + 5000; //5000 to 8000 ms
	}
	else
	{
		mstowait = (rand()% 3000) + 500; // 500 to 5000 ms
	}
	while(mstowait > 0)
	{
		mstowait--;
		if(GetButtonInputP1() != 0 && *p1_commit == FALSE)
		{
			Sound_Commit();
			*p1_commit = TRUE; //foul
		}
		if(GetButtonInputP2() != 0 && *p2_commit == FALSE)
		{
			Sound_Commit();
			*p2_commit = TRUE; //foul
		}
		_delay_ms(1);
	}
}

void LEDControl (int selection)
{
	switch(selection)
	{
	case 0: { PORTD |= 0b00100001;break; } //PD0 PD5 (BA) is 11 (No LED)
	case 1: { PORTD &= 0b11011110;break; } //PD0 PD5 (BA) is 00 (LED1)
	case 2: { PORTD &= 0b11111110; PORTD |= 0b00100000;break; } //PD0 PD5 (BA) is 01 (LED1)
	case 3: { PORTD &= 0b11011111; PORTD |= 0b00000001;break; } //PD0 PD5 (BA) is 10 (LED2)

	default : { PORTD |= 0b00100001;break; } //PD0 PD5 (BA) is 11 (No LED)
	}
}

void SevenSegmentP1(int health)
{
	//PB5 = BCDSeven0_P1 (LSB)
	//PB3 = BCDSeven1_P1
	//PB1 = BCDSeven2_P1 (MSB)
	
	PORTB |= 0b00101000;
	PORTB &= 0b11111101;
	_delay_ms(30);
	PORTB |= 0b00100000;
	PORTB &= 0b11110101;
	_delay_ms(30);
	PORTB |= 0b00101010;
	PORTB &= 0b11111111;
	_delay_ms(30);
	PORTB |= 0b00000010;
	PORTB &= 0b11010111;
	_delay_ms(30);
	PORTB |= 0b00001000;
	PORTB &= 0b11011101;
	_delay_ms(30);	
	
	switch(health)
	{
	case 0: //000
		{
			PORTB |= 0b00000000;
			PORTB &= 0b11010101;break;
		}
	case 1: //001
		{
			PORTB |= 0b00100000;
			PORTB &= 0b11110101;break;
		}
	case 2: //010
		{
			PORTB |= 0b00000010;
			PORTB &= 0b11010111;break;
		}
	case 3: //011
		{
			PORTB |= 0b00100010;
			PORTB &= 0b11110111;break;
		}
	case 4: //100
		{
			PORTB |= 0b00001000;
			PORTB &= 0b11011101;break;
		}
	case 5: //101
		{
			PORTB |= 0b00101000;
			PORTB &= 0b11111101;break;
		}
	case 6: //110
		{
			PORTB |= 0b00001010;
			PORTB &= 0b11011111;break;
		}
	case 7: //111
		{
			PORTB |= 0b00101010;
			PORTB &= 0b11111111;break;
		}
	default : //000
		{
			PORTB |= 0b00000000;
			PORTB &= 0b11010101;break;
		}
	}
	_delay_ms(400);
}

void SevenSegmentP2(int health)
{
	//PC5 = BCDSeven0_P2 (LSB)
	//PC3 = BCDSeven1_P2
	//PC1 = BCDSeven2_P2 (MSB)
	PORTC |= 0b00101000;
	PORTC &= 0b11111101;
	_delay_ms(30);
	PORTC |= 0b00100000;
	PORTC &= 0b11110101;
	_delay_ms(30);
	PORTC |= 0b00101010;
	PORTC &= 0b11111111;
	_delay_ms(30);
	PORTC |= 0b00000010;
	PORTC &= 0b11010111;
	_delay_ms(30);
	PORTC |= 0b00001000;
	PORTC &= 0b11011101;
	_delay_ms(30);	
	switch(health)
	{
	case 0: //000
		{
			PORTC |= 0b00000000;
			PORTC &= 0b11010101;break;
		}
	case 1: //001
		{
			PORTC |= 0b00100000;
			PORTC &= 0b11110101;break;
		}
	case 2: //010
		{
			PORTC |= 0b00000010;
			PORTC &= 0b11010111;break;
		}
	case 3: //011
		{
			PORTC |= 0b00100010;
			PORTC &= 0b11110111;break;
		}
	case 4: //100
		{
			PORTC |= 0b00001000;
			PORTC &= 0b11011101;break;
		}
	case 5: //101
		{
			PORTC |= 0b00101000;
			PORTC &= 0b11111101;break;
		}
	case 6: //110
		{
			PORTC |= 0b00001010;
			PORTC &= 0b11011111;break;
		}
	case 7: //111
		{
			PORTC |= 0b00101010;
			PORTC &= 0b11111111;break;
		}
	default : //000
		{
			PORTC |= 0b00000000;
			PORTC &= 0b11010101;break;
		}
	}
	_delay_ms(400);
}


void RandomDisplayLED(int *active_led)
{
	*active_led = ( rand()% 3) + 1; // random 1 2 3
	Sound_Appear();
	switch(*active_led)
	{
	case 1: { LEDControl(1); break;}
	case 2: { LEDControl(2); break;}
	case 3: { LEDControl(3); break;}
	default : { LEDControl(0); break;} // error?
	}
}

void Wait_Strike (int *p1_life,int *p2_life,int *p1_commit,int *p2_commit,int active_led)
{
	int p1_choice = 0;
	int p2_choice = 0;
	if(*p1_commit == TRUE && *p2_commit == TRUE) // both foul before strike
	{	
		LEDControl(0); // off all led
		*p1_commit = FALSE;
		*p2_commit = FALSE;	
		return;
	}
	
	while(TRUE)
	{
		if(GetButtonInputP1() != 0 && GetButtonInputP1() == active_led && *p1_commit == FALSE) // P1 correct strike
		{
			LEDControl(0); // off all led
			*p1_commit = TRUE;
			*p2_life -= 1; //damage
			Sound_Damage();
			SevenSegmentP2(*p2_life);
			break;
		}
		if(GetButtonInputP2() != 0 && GetButtonInputP2() == active_led && *p2_commit == FALSE) // P2 correct strike
		{
			LEDControl(0); // off all led
			*p2_commit = TRUE;
			*p1_life -= 1; //damage
			Sound_Damage();
			SevenSegmentP1(*p1_life);
			break;
		}
		if(GetButtonInputP1() != 0 && GetButtonInputP1() != active_led && *p1_commit == FALSE) // P1 wrong strike
		{
			Sound_Commit();
			*p1_commit = TRUE;
		}
		if(GetButtonInputP2() != 0 && GetButtonInputP2() != active_led && *p2_commit == FALSE) // P1 wrong strike
		{
			Sound_Commit();
			*p2_commit = TRUE;
		}
		if(*p1_commit == TRUE && *p2_commit == TRUE) // no one hurt
		{
			LEDControl(0); // off all led
			_delay_ms(500);
			break;
		}
		///////////////////////////////////////////////////////////////////////////////
	}
	// Reset Commit
	*p1_commit = FALSE;
	*p2_commit = FALSE;

}

void EndingLoop()
{
	Sound_Ending();
	while(TRUE)
	{
		LEDControl(1);
		_delay_ms(300);
		LEDControl(2);
		_delay_ms(300);
		LEDControl(3);
		_delay_ms(300);
		LEDControl(2);
		_delay_ms(300);
		if(GetButtonInputP1() != 0 || GetButtonInputP2() != 0)
		{
			break;
		}
	}
}
void updateLife(int P1_life,int P2_life)
{
	SevenSegmentP1(P1_life);
	SevenSegmentP2(P2_life);
}

void main()
{
	int instantmode;
	int p1_life;
	int p2_life;
	int p1_commit=FALSE;
	int p2_commit=FALSE;
	int active_led = 0;
	

	
	while(TRUE)
	{
		InitDDR();
		LEDControl(0);
		
		instantmode = ReceiveMode();

		if(instantmode)
		{
			p1_life = 1;
			p2_life = 1;	
		}
		else
		{
			p1_life = 7;
			p2_life = 7;
		}
		Sound_Start();
		updateLife(p1_life,p2_life);
		while (p1_life != 0 && p2_life != 0)
		{
			Wait_LED(instantmode,&p1_commit,&p2_commit);
			RandomDisplayLED(&active_led);
			Wait_Strike(&p1_life,&p2_life,&p1_commit,&p2_commit,active_led);
			
			active_led = 0; // this too
		}
		
		EndingLoop();
	}
}
